// $Id: TGo4EventSourceParameter.h 478 2009-10-29 12:26:09Z linev $
//-----------------------------------------------------------------------
//       The GSI Online Offline Object Oriented (Go4) Project
//         Experiment Data Processing at EE department, GSI
//-----------------------------------------------------------------------
// Copyright (C) 2000- GSI Helmholtzzentrum fr Schwerionenforschung GmbH
//                     Planckstr. 1, 64291 Darmstadt, Germany
// Contact:            http://go4.gsi.de
//-----------------------------------------------------------------------
// This software can be used under the license agreements as stated
// in Go4License.txt file which is part of the distribution.
//-----------------------------------------------------------------------

#ifndef TGO4EVENTSOURCEPARAMETER_H
#define TGO4EVENTSOURCEPARAMETER_H

#include "TGo4Parameter.h"

/**
 * Basic type for all classes containing information
 * to parameterize the event source. For use in the
 * event factory; factory method gets (possibly subclassed)
 * parameter object which is suited to the concrete user
 * factory. Name string can be used as filename, e.g.
 */
class TGo4EventSourceParameter : public TGo4Parameter {
   public:
      TGo4EventSourceParameter();

      TGo4EventSourceParameter(const char* name, Int_t id = 0);

      virtual ~TGo4EventSourceParameter() ;

      Int_t GetID() const { return fiID; }

      Int_t GetTimeout() const { return fiTimeout; }

      void SetTimeout(Int_t out) { fiTimeout=out; }

      /** basic method to printout status information
        * on stdout; to be overridden by specific subclass */
      virtual Int_t PrintParameter(Text_t* buffer=0, Int_t buflen=0);

      /** update contents of paramter class with external object. */
      virtual Bool_t UpdateFrom(TGo4Parameter* rhs);

   private:

      /** Identity number to specify the kind. Optional, default is 0. */
      Int_t fiID;

      /** Timeout in seconds. If -1, no timeout (default)  */
      Int_t fiTimeout;

   ClassDef(TGo4EventSourceParameter,1)
};

#endif //TGO4EVENTSOURCEPARAMETER_H
